from typing import TypedDict, Literal, Any, Coroutine, Union, overload, Generator, final
from typing_extensions import NotRequired
from hfortix_fortios.models import FortiObject, FortiObjectList

# ============================================================================
# Nested TypedDicts for table field children (dict mode)
# These MUST be defined before the Payload class to use them as type hints
# ============================================================================

# ============================================================================
# Payload TypedDict for IDE autocomplete (for POST/PUT - fields are optional)
# ============================================================================
# NOTE: We intentionally DON'T use NotRequired wrapper because:
# 1. total=False already makes all fields optional
# 2. NotRequired[Literal[...]] prevents Pylance from validating Literal values in dict literals
class SettingPayload(TypedDict, total=False):
    """
    Type hints for vpn/certificate/setting payload fields.
    
    VPN certificate setting.
    
    **Related Resources:**

    Dependencies (resources this endpoint references):
        - :class:`~.system.interface.InterfaceEndpoint` (via: interface)
        - :class:`~.vpn.certificate.local.LocalEndpoint` (via: certname-dsa1024, certname-dsa2048, certname-ecdsa256, +7 more)
        - :class:`~.vpn.certificate.ocsp-server.OcspServerEndpoint` (via: ocsp-default-server)

    **Usage:**
        payload: SettingPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    ocsp_status: Literal["enable", "mandatory", "disable"]  # Enable/disable receiving certificates using the OC | Default: disable
    ocsp_option: Literal["certificate", "server"]  # Specify whether the OCSP URL is from certificate o | Default: server
    proxy: str  # Proxy server FQDN or IP for OCSP/CA queries during | MaxLen: 127
    proxy_port: int  # Proxy server port (1 - 65535, default = 8080). | Default: 8080 | Min: 1 | Max: 65535
    proxy_username: str  # Proxy server user name. | MaxLen: 63
    proxy_password: str  # Proxy server password. | MaxLen: 128
    source_ip: str  # Source IP address for dynamic AIA and OCSP queries | MaxLen: 63
    ocsp_default_server: str  # Default OCSP server. | MaxLen: 35
    interface_select_method: Literal["auto", "sdwan", "specify"]  # Specify how to select outgoing interface to reach | Default: auto
    interface: str  # Specify outgoing interface to reach server. | MaxLen: 15
    vrf_select: int  # VRF ID used for connection to server. | Default: 0 | Min: 0 | Max: 511
    check_ca_cert: Literal["enable", "disable"]  # Enable/disable verification of the user certificat | Default: enable
    check_ca_chain: Literal["enable", "disable"]  # Enable/disable verification of the entire certific | Default: disable
    subject_match: Literal["substring", "value"]  # When searching for a matching certificate, control | Default: substring
    subject_set: Literal["subset", "superset"]  # When searching for a matching certificate, control | Default: subset
    cn_match: Literal["substring", "value"]  # When searching for a matching certificate, control | Default: substring
    cn_allow_multi: Literal["disable", "enable"]  # When searching for a matching certificate, allow m | Default: enable
    crl_verification: str  # CRL verification options.
    strict_ocsp_check: Literal["enable", "disable"]  # Enable/disable strict mode OCSP checking. | Default: disable
    ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]  # Minimum supported protocol version for SSL/TLS con | Default: default
    cmp_save_extra_certs: Literal["enable", "disable"]  # Enable/disable saving extra certificates in CMP mo | Default: disable
    cmp_key_usage_checking: Literal["enable", "disable"]  # Enable/disable server certificate key usage checki | Default: enable
    cert_expire_warning: int  # Number of days before a certificate expires to sen | Default: 14 | Min: 0 | Max: 100
    certname_rsa1024: str  # 1024 bit RSA key certificate for re-signing server | Default: Fortinet_SSL_RSA1024 | MaxLen: 35
    certname_rsa2048: str  # 2048 bit RSA key certificate for re-signing server | Default: Fortinet_SSL_RSA2048 | MaxLen: 35
    certname_rsa4096: str  # 4096 bit RSA key certificate for re-signing server | Default: Fortinet_SSL_RSA4096 | MaxLen: 35
    certname_dsa1024: str  # 1024 bit DSA key certificate for re-signing server | Default: Fortinet_SSL_DSA1024 | MaxLen: 35
    certname_dsa2048: str  # 2048 bit DSA key certificate for re-signing server | Default: Fortinet_SSL_DSA2048 | MaxLen: 35
    certname_ecdsa256: str  # 256 bit ECDSA key certificate for re-signing serve | Default: Fortinet_SSL_ECDSA256 | MaxLen: 35
    certname_ecdsa384: str  # 384 bit ECDSA key certificate for re-signing serve | Default: Fortinet_SSL_ECDSA384 | MaxLen: 35
    certname_ecdsa521: str  # 521 bit ECDSA key certificate for re-signing serve | Default: Fortinet_SSL_ECDSA521 | MaxLen: 35
    certname_ed25519: str  # 253 bit EdDSA key certificate for re-signing serve | Default: Fortinet_SSL_ED25519 | MaxLen: 35
    certname_ed448: str  # 456 bit EdDSA key certificate for re-signing serve | Default: Fortinet_SSL_ED448 | MaxLen: 35

# ============================================================================
# Nested classes for table field children (object mode - for API responses)
# ============================================================================



# Response TypedDict for GET returns (all fields present in API response)
class SettingResponse(TypedDict):
    """
    Type hints for vpn/certificate/setting API response fields.
    
    All fields are present in the response from the FortiGate API.
    """
    ocsp_status: Literal["enable", "mandatory", "disable"]  # Enable/disable receiving certificates using the OC | Default: disable
    ocsp_option: Literal["certificate", "server"]  # Specify whether the OCSP URL is from certificate o | Default: server
    proxy: str  # Proxy server FQDN or IP for OCSP/CA queries during | MaxLen: 127
    proxy_port: int  # Proxy server port (1 - 65535, default = 8080). | Default: 8080 | Min: 1 | Max: 65535
    proxy_username: str  # Proxy server user name. | MaxLen: 63
    proxy_password: str  # Proxy server password. | MaxLen: 128
    source_ip: str  # Source IP address for dynamic AIA and OCSP queries | MaxLen: 63
    ocsp_default_server: str  # Default OCSP server. | MaxLen: 35
    interface_select_method: Literal["auto", "sdwan", "specify"]  # Specify how to select outgoing interface to reach | Default: auto
    interface: str  # Specify outgoing interface to reach server. | MaxLen: 15
    vrf_select: int  # VRF ID used for connection to server. | Default: 0 | Min: 0 | Max: 511
    check_ca_cert: Literal["enable", "disable"]  # Enable/disable verification of the user certificat | Default: enable
    check_ca_chain: Literal["enable", "disable"]  # Enable/disable verification of the entire certific | Default: disable
    subject_match: Literal["substring", "value"]  # When searching for a matching certificate, control | Default: substring
    subject_set: Literal["subset", "superset"]  # When searching for a matching certificate, control | Default: subset
    cn_match: Literal["substring", "value"]  # When searching for a matching certificate, control | Default: substring
    cn_allow_multi: Literal["disable", "enable"]  # When searching for a matching certificate, allow m | Default: enable
    crl_verification: str  # CRL verification options.
    strict_ocsp_check: Literal["enable", "disable"]  # Enable/disable strict mode OCSP checking. | Default: disable
    ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]  # Minimum supported protocol version for SSL/TLS con | Default: default
    cmp_save_extra_certs: Literal["enable", "disable"]  # Enable/disable saving extra certificates in CMP mo | Default: disable
    cmp_key_usage_checking: Literal["enable", "disable"]  # Enable/disable server certificate key usage checki | Default: enable
    cert_expire_warning: int  # Number of days before a certificate expires to sen | Default: 14 | Min: 0 | Max: 100
    certname_rsa1024: str  # 1024 bit RSA key certificate for re-signing server | Default: Fortinet_SSL_RSA1024 | MaxLen: 35
    certname_rsa2048: str  # 2048 bit RSA key certificate for re-signing server | Default: Fortinet_SSL_RSA2048 | MaxLen: 35
    certname_rsa4096: str  # 4096 bit RSA key certificate for re-signing server | Default: Fortinet_SSL_RSA4096 | MaxLen: 35
    certname_dsa1024: str  # 1024 bit DSA key certificate for re-signing server | Default: Fortinet_SSL_DSA1024 | MaxLen: 35
    certname_dsa2048: str  # 2048 bit DSA key certificate for re-signing server | Default: Fortinet_SSL_DSA2048 | MaxLen: 35
    certname_ecdsa256: str  # 256 bit ECDSA key certificate for re-signing serve | Default: Fortinet_SSL_ECDSA256 | MaxLen: 35
    certname_ecdsa384: str  # 384 bit ECDSA key certificate for re-signing serve | Default: Fortinet_SSL_ECDSA384 | MaxLen: 35
    certname_ecdsa521: str  # 521 bit ECDSA key certificate for re-signing serve | Default: Fortinet_SSL_ECDSA521 | MaxLen: 35
    certname_ed25519: str  # 253 bit EdDSA key certificate for re-signing serve | Default: Fortinet_SSL_ED25519 | MaxLen: 35
    certname_ed448: str  # 456 bit EdDSA key certificate for re-signing serve | Default: Fortinet_SSL_ED448 | MaxLen: 35


@final
class SettingObject:
    """Typed FortiObject for vpn/certificate/setting with IDE autocomplete support.
    
    This is a typed wrapper that provides IDE autocomplete for API response fields.
    At runtime, this is actually a FortiObject instance.
    """
    
    # Enable/disable receiving certificates using the OCSP. | Default: disable
    ocsp_status: Literal["enable", "mandatory", "disable"]
    # Specify whether the OCSP URL is from certificate or configur | Default: server
    ocsp_option: Literal["certificate", "server"]
    # Proxy server FQDN or IP for OCSP/CA queries during certifica | MaxLen: 127
    proxy: str
    # Proxy server port (1 - 65535, default = 8080). | Default: 8080 | Min: 1 | Max: 65535
    proxy_port: int
    # Proxy server user name. | MaxLen: 63
    proxy_username: str
    # Proxy server password. | MaxLen: 128
    proxy_password: str
    # Source IP address for dynamic AIA and OCSP queries. | MaxLen: 63
    source_ip: str
    # Default OCSP server. | MaxLen: 35
    ocsp_default_server: str
    # Specify how to select outgoing interface to reach server. | Default: auto
    interface_select_method: Literal["auto", "sdwan", "specify"]
    # Specify outgoing interface to reach server. | MaxLen: 15
    interface: str
    # VRF ID used for connection to server. | Default: 0 | Min: 0 | Max: 511
    vrf_select: int
    # Enable/disable verification of the user certificate and pass | Default: enable
    check_ca_cert: Literal["enable", "disable"]
    # Enable/disable verification of the entire certificate chain | Default: disable
    check_ca_chain: Literal["enable", "disable"]
    # When searching for a matching certificate, control how to do | Default: substring
    subject_match: Literal["substring", "value"]
    # When searching for a matching certificate, control how to do | Default: subset
    subject_set: Literal["subset", "superset"]
    # When searching for a matching certificate, control how to do | Default: substring
    cn_match: Literal["substring", "value"]
    # When searching for a matching certificate, allow multiple CN | Default: enable
    cn_allow_multi: Literal["disable", "enable"]
    # CRL verification options.
    crl_verification: str
    # Enable/disable strict mode OCSP checking. | Default: disable
    strict_ocsp_check: Literal["enable", "disable"]
    # Minimum supported protocol version for SSL/TLS connections | Default: default
    ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]
    # Enable/disable saving extra certificates in CMP mode | Default: disable
    cmp_save_extra_certs: Literal["enable", "disable"]
    # Enable/disable server certificate key usage checking in CMP | Default: enable
    cmp_key_usage_checking: Literal["enable", "disable"]
    # Number of days before a certificate expires to send a warnin | Default: 14 | Min: 0 | Max: 100
    cert_expire_warning: int
    # 1024 bit RSA key certificate for re-signing server certifica | Default: Fortinet_SSL_RSA1024 | MaxLen: 35
    certname_rsa1024: str
    # 2048 bit RSA key certificate for re-signing server certifica | Default: Fortinet_SSL_RSA2048 | MaxLen: 35
    certname_rsa2048: str
    # 4096 bit RSA key certificate for re-signing server certifica | Default: Fortinet_SSL_RSA4096 | MaxLen: 35
    certname_rsa4096: str
    # 1024 bit DSA key certificate for re-signing server certifica | Default: Fortinet_SSL_DSA1024 | MaxLen: 35
    certname_dsa1024: str
    # 2048 bit DSA key certificate for re-signing server certifica | Default: Fortinet_SSL_DSA2048 | MaxLen: 35
    certname_dsa2048: str
    # 256 bit ECDSA key certificate for re-signing server certific | Default: Fortinet_SSL_ECDSA256 | MaxLen: 35
    certname_ecdsa256: str
    # 384 bit ECDSA key certificate for re-signing server certific | Default: Fortinet_SSL_ECDSA384 | MaxLen: 35
    certname_ecdsa384: str
    # 521 bit ECDSA key certificate for re-signing server certific | Default: Fortinet_SSL_ECDSA521 | MaxLen: 35
    certname_ecdsa521: str
    # 253 bit EdDSA key certificate for re-signing server certific | Default: Fortinet_SSL_ED25519 | MaxLen: 35
    certname_ed25519: str
    # 456 bit EdDSA key certificate for re-signing server certific | Default: Fortinet_SSL_ED448 | MaxLen: 35
    certname_ed448: str
    
    # Common API response fields
    status: str
    http_status: int | None
    http_status_code: int | None
    http_method: str | None
    http_response_time: float | None
    vdom: str | None
    
    # Methods from FortiObject
    @property
    def dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        ...
    @property
    def json(self) -> str:
        """Get pretty-printed JSON string."""
        ...
    @property
    def raw(self) -> dict[str, Any]:
        """Get raw API response data."""
        ...
    def get_full(self, name: str) -> Any: ...
    def to_dict(self) -> SettingPayload: ...
    def keys(self) -> Any: ...
    def values(self) -> Generator[Any, None, None]: ...
    def items(self) -> Generator[tuple[str, Any], None, None]: ...
    def get(self, key: str, default: Any = None) -> Any: ...


class Setting:
    """
    VPN certificate setting.
    
    Path: vpn/certificate/setting
    Category: cmdb
    """
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client.
        
        Args:
            client: HTTP client instance for API communication
        """
        ...
    
    # ================================================================
    # GET OVERLOADS - Always returns FortiObject
    # Pylance matches overloads top-to-bottom, so these must come first!
    # ================================================================
    
    # With mkey as positional arg -> returns FortiObject
    @overload
    def get(
        self,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # With mkey as keyword arg -> returns FortiObject
    @overload
    def get(
        self,
        *,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Without mkey -> returns list of FortiObjects
    @overload
    def get(
        self,
        name: None = None,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # ================================================================
    # (removed - all GET now returns FortiObject)
    # ================================================================
    
    # With mkey as positional arg -> returns single object
    @overload
    def get(
        self,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # With mkey as keyword arg -> returns single object
    @overload
    def get(
        self,
        *,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # With no mkey -> returns list of objects
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Dict mode with mkey provided as positional arg (single dict)
    @overload
    def get(
        self,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Dict mode with mkey provided as keyword arg (single dict)
    @overload
    def get(
        self,
        *,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Dict mode - list of dicts (no mkey/name provided) - keyword-only signature
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Fallback overload for all other cases
    @overload
    def get(
        self,
        name: str | None = ...,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> dict[str, Any] | FortiObject: ...
    
    def get(
        self,
        name: str | None = ...,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject | dict[str, Any]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...
    
    # PUT overloads
    @overload
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        ocsp_status: Literal["enable", "mandatory", "disable"] | None = ...,
        ocsp_option: Literal["certificate", "server"] | None = ...,
        proxy: str | None = ...,
        proxy_port: int | None = ...,
        proxy_username: str | None = ...,
        proxy_password: str | None = ...,
        source_ip: str | None = ...,
        ocsp_default_server: str | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        check_ca_cert: Literal["enable", "disable"] | None = ...,
        check_ca_chain: Literal["enable", "disable"] | None = ...,
        subject_match: Literal["substring", "value"] | None = ...,
        subject_set: Literal["subset", "superset"] | None = ...,
        cn_match: Literal["substring", "value"] | None = ...,
        cn_allow_multi: Literal["disable", "enable"] | None = ...,
        crl_verification: str | None = ...,
        strict_ocsp_check: Literal["enable", "disable"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        cmp_save_extra_certs: Literal["enable", "disable"] | None = ...,
        cmp_key_usage_checking: Literal["enable", "disable"] | None = ...,
        cert_expire_warning: int | None = ...,
        certname_rsa1024: str | None = ...,
        certname_rsa2048: str | None = ...,
        certname_rsa4096: str | None = ...,
        certname_dsa1024: str | None = ...,
        certname_dsa2048: str | None = ...,
        certname_ecdsa256: str | None = ...,
        certname_ecdsa384: str | None = ...,
        certname_ecdsa521: str | None = ...,
        certname_ed25519: str | None = ...,
        certname_ed448: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    @overload
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        ocsp_status: Literal["enable", "mandatory", "disable"] | None = ...,
        ocsp_option: Literal["certificate", "server"] | None = ...,
        proxy: str | None = ...,
        proxy_port: int | None = ...,
        proxy_username: str | None = ...,
        proxy_password: str | None = ...,
        source_ip: str | None = ...,
        ocsp_default_server: str | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        check_ca_cert: Literal["enable", "disable"] | None = ...,
        check_ca_chain: Literal["enable", "disable"] | None = ...,
        subject_match: Literal["substring", "value"] | None = ...,
        subject_set: Literal["subset", "superset"] | None = ...,
        cn_match: Literal["substring", "value"] | None = ...,
        cn_allow_multi: Literal["disable", "enable"] | None = ...,
        crl_verification: str | None = ...,
        strict_ocsp_check: Literal["enable", "disable"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        cmp_save_extra_certs: Literal["enable", "disable"] | None = ...,
        cmp_key_usage_checking: Literal["enable", "disable"] | None = ...,
        cert_expire_warning: int | None = ...,
        certname_rsa1024: str | None = ...,
        certname_rsa2048: str | None = ...,
        certname_rsa4096: str | None = ...,
        certname_dsa1024: str | None = ...,
        certname_dsa2048: str | None = ...,
        certname_ecdsa256: str | None = ...,
        certname_ecdsa384: str | None = ...,
        certname_ecdsa521: str | None = ...,
        certname_ed25519: str | None = ...,
        certname_ed448: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # Default overload
    @overload
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        ocsp_status: Literal["enable", "mandatory", "disable"] | None = ...,
        ocsp_option: Literal["certificate", "server"] | None = ...,
        proxy: str | None = ...,
        proxy_port: int | None = ...,
        proxy_username: str | None = ...,
        proxy_password: str | None = ...,
        source_ip: str | None = ...,
        ocsp_default_server: str | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        check_ca_cert: Literal["enable", "disable"] | None = ...,
        check_ca_chain: Literal["enable", "disable"] | None = ...,
        subject_match: Literal["substring", "value"] | None = ...,
        subject_set: Literal["subset", "superset"] | None = ...,
        cn_match: Literal["substring", "value"] | None = ...,
        cn_allow_multi: Literal["disable", "enable"] | None = ...,
        crl_verification: str | None = ...,
        strict_ocsp_check: Literal["enable", "disable"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        cmp_save_extra_certs: Literal["enable", "disable"] | None = ...,
        cmp_key_usage_checking: Literal["enable", "disable"] | None = ...,
        cert_expire_warning: int | None = ...,
        certname_rsa1024: str | None = ...,
        certname_rsa2048: str | None = ...,
        certname_rsa4096: str | None = ...,
        certname_dsa1024: str | None = ...,
        certname_dsa2048: str | None = ...,
        certname_ecdsa256: str | None = ...,
        certname_ecdsa384: str | None = ...,
        certname_ecdsa521: str | None = ...,
        certname_ed25519: str | None = ...,
        certname_ed448: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        ocsp_status: Literal["enable", "mandatory", "disable"] | None = ...,
        ocsp_option: Literal["certificate", "server"] | None = ...,
        proxy: str | None = ...,
        proxy_port: int | None = ...,
        proxy_username: str | None = ...,
        proxy_password: str | None = ...,
        source_ip: str | None = ...,
        ocsp_default_server: str | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        check_ca_cert: Literal["enable", "disable"] | None = ...,
        check_ca_chain: Literal["enable", "disable"] | None = ...,
        subject_match: Literal["substring", "value"] | None = ...,
        subject_set: Literal["subset", "superset"] | None = ...,
        cn_match: Literal["substring", "value"] | None = ...,
        cn_allow_multi: Literal["disable", "enable"] | None = ...,
        crl_verification: str | None = ...,
        strict_ocsp_check: Literal["enable", "disable"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        cmp_save_extra_certs: Literal["enable", "disable"] | None = ...,
        cmp_key_usage_checking: Literal["enable", "disable"] | None = ...,
        cert_expire_warning: int | None = ...,
        certname_rsa1024: str | None = ...,
        certname_rsa2048: str | None = ...,
        certname_rsa4096: str | None = ...,
        certname_dsa1024: str | None = ...,
        certname_dsa2048: str | None = ...,
        certname_ecdsa256: str | None = ...,
        certname_ecdsa384: str | None = ...,
        certname_ecdsa521: str | None = ...,
        certname_ed25519: str | None = ...,
        certname_ed448: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: SettingPayload | None = ...,
        ocsp_status: Literal["enable", "mandatory", "disable"] | None = ...,
        ocsp_option: Literal["certificate", "server"] | None = ...,
        proxy: str | None = ...,
        proxy_port: int | None = ...,
        proxy_username: str | None = ...,
        proxy_password: str | None = ...,
        source_ip: str | None = ...,
        ocsp_default_server: str | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        check_ca_cert: Literal["enable", "disable"] | None = ...,
        check_ca_chain: Literal["enable", "disable"] | None = ...,
        subject_match: Literal["substring", "value"] | None = ...,
        subject_set: Literal["subset", "superset"] | None = ...,
        cn_match: Literal["substring", "value"] | None = ...,
        cn_allow_multi: Literal["disable", "enable"] | None = ...,
        crl_verification: str | None = ...,
        strict_ocsp_check: Literal["enable", "disable"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        cmp_save_extra_certs: Literal["enable", "disable"] | None = ...,
        cmp_key_usage_checking: Literal["enable", "disable"] | None = ...,
        cert_expire_warning: int | None = ...,
        certname_rsa1024: str | None = ...,
        certname_rsa2048: str | None = ...,
        certname_rsa4096: str | None = ...,
        certname_dsa1024: str | None = ...,
        certname_dsa2048: str | None = ...,
        certname_ecdsa256: str | None = ...,
        certname_ecdsa384: str | None = ...,
        certname_ecdsa521: str | None = ...,
        certname_ed25519: str | None = ...,
        certname_ed448: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


# ================================================================


__all__ = [
    "Setting",
    "SettingPayload",
    "SettingResponse",
    "SettingObject",
]


"""
Pydantic Models for CMDB - log/threat_weight

Runtime validation models for log/threat_weight configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Child Table Models
# ============================================================================

class ThreatWeightLevel(BaseModel):
    """
    Child table model for level.
    
    Score mapping for threat weight levels.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    low: int | None = Field(ge=1, le=100, default=5, description="Low level score value (1 - 100).")    
    medium: int | None = Field(ge=1, le=100, default=10, description="Medium level score value (1 - 100).")    
    high: int | None = Field(ge=1, le=100, default=30, description="High level score value (1 - 100).")    
    critical: int | None = Field(ge=1, le=100, default=50, description="Critical level score value (1 - 100).")
class ThreatWeightMalware(BaseModel):
    """
    Child table model for malware.
    
    Anti-virus malware threat weight settings.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    virus_infected: VirusInfectedEnum | None = Field(default="critical", description="Threat weight score for virus (infected) detected.")    
    inline_block: InlineBlockEnum | None = Field(default="critical", description="Threat weight score for malware detected by inline block.")    
    file_blocked: FileBlockedEnum | None = Field(default="low", description="Threat weight score for blocked file detected.")    
    command_blocked: CommandBlockedEnum | None = Field(default="disable", description="Threat weight score for blocked command detected.")    
    oversized: OversizedEnum | None = Field(default="disable", description="Threat weight score for oversized file detected.")    
    virus_scan_error: VirusScanErrorEnum | None = Field(default="high", description="Threat weight score for virus (scan error) detected.")    
    switch_proto: SwitchProtoEnum | None = Field(default="disable", description="Threat weight score for switch proto detected.")    
    mimefragmented: MimefragmentedEnum | None = Field(default="disable", description="Threat weight score for mimefragmented detected.")    
    virus_file_type_executable: VirusFileTypeExecutableEnum | None = Field(default="medium", description="Threat weight score for virus (file type executable) detected.")    
    virus_outbreak_prevention: VirusOutbreakPreventionEnum | None = Field(default="critical", description="Threat weight score for virus (outbreak prevention) event.")    
    content_disarm: ContentDisarmEnum | None = Field(default="medium", description="Threat weight score for virus (content disarm) detected.")    
    malware_list: MalwareListEnum | None = Field(default="medium", description="Threat weight score for virus (malware list) detected.")    
    ems_threat_feed: EmsThreatFeedEnum | None = Field(default="medium", description="Threat weight score for virus (EMS threat feed) detected.")    
    fsa_malicious: FsaMaliciousEnum | None = Field(default="critical", description="Threat weight score for FortiSandbox malicious malware detected.")    
    fsa_high_risk: FsaHighRiskEnum | None = Field(default="high", description="Threat weight score for FortiSandbox high risk malware detected.")    
    fsa_medium_risk: FsaMediumRiskEnum | None = Field(default="medium", description="Threat weight score for FortiSandbox medium risk malware detected.")
class ThreatWeightIps(BaseModel):
    """
    Child table model for ips.
    
    IPS threat weight settings.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    info_severity: InfoSeverityEnum | None = Field(default="disable", description="Threat weight score for IPS info severity events.")    
    low_severity: LowSeverityEnum | None = Field(default="low", description="Threat weight score for IPS low severity events.")    
    medium_severity: MediumSeverityEnum | None = Field(default="medium", description="Threat weight score for IPS medium severity events.")    
    high_severity: HighSeverityEnum | None = Field(default="high", description="Threat weight score for IPS high severity events.")    
    critical_severity: CriticalSeverityEnum | None = Field(default="critical", description="Threat weight score for IPS critical severity events.")
class ThreatWeightWeb(BaseModel):
    """
    Child table model for web.
    
    Web filtering threat weight settings.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    id: int | None = Field(ge=0, le=255, default=0, description="Entry ID.")    
    category: int = Field(ge=0, le=255, default=0, description="Threat weight score for web category filtering matches.")    
    level: LevelEnum | None = Field(default="low", description="Threat weight score for web category filtering matches.")
class ThreatWeightGeolocation(BaseModel):
    """
    Child table model for geolocation.
    
    Geolocation-based threat weight settings.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    id: int | None = Field(ge=0, le=255, default=0, description="Entry ID.")    
    country: str = Field(max_length=2, default="", description="Country code.")    
    level: LevelEnum | None = Field(default="low", description="Threat weight score for Geolocation-based events.")
class ThreatWeightApplication(BaseModel):
    """
    Child table model for application.
    
    Application-control threat weight settings.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    id: int | None = Field(ge=0, le=255, default=0, description="Entry ID.")    
    category: int = Field(ge=0, le=65535, default=0, description="Application category.")    
    level: LevelEnum | None = Field(default="low", description="Threat weight score for Application events.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class ThreatWeightBlocked_connectionEnum(str, Enum):
    """Allowed values for blocked_connection field."""
    DISABLE = "disable"    LOW = "low"    MEDIUM = "medium"    HIGH = "high"    CRITICAL = "critical"
class ThreatWeightFailed_connectionEnum(str, Enum):
    """Allowed values for failed_connection field."""
    DISABLE = "disable"    LOW = "low"    MEDIUM = "medium"    HIGH = "high"    CRITICAL = "critical"
class ThreatWeightUrl_block_detectedEnum(str, Enum):
    """Allowed values for url_block_detected field."""
    DISABLE = "disable"    LOW = "low"    MEDIUM = "medium"    HIGH = "high"    CRITICAL = "critical"
class ThreatWeightBotnet_connection_detectedEnum(str, Enum):
    """Allowed values for botnet_connection_detected field."""
    DISABLE = "disable"    LOW = "low"    MEDIUM = "medium"    HIGH = "high"    CRITICAL = "critical"

# ============================================================================
# Main Model
# ============================================================================

class ThreatWeightModel(BaseModel):
    """
    Pydantic model for log/threat_weight configuration.
    
    Configure threat weight settings.
    
    Validation Rules:        - status: pattern=        - level: pattern=        - blocked_connection: pattern=        - failed_connection: pattern=        - url_block_detected: pattern=        - botnet_connection_detected: pattern=        - malware: pattern=        - ips: pattern=        - web: pattern=        - geolocation: pattern=        - application: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    status: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable the threat weight feature.")    
    level: list[Level] = Field(default=None, description="Score mapping for threat weight levels.")    
    blocked_connection: BlockedConnectionEnum | None = Field(default="high", description="Threat weight score for blocked connections.")    
    failed_connection: FailedConnectionEnum | None = Field(default="low", description="Threat weight score for failed connections.")    
    url_block_detected: UrlBlockDetectedEnum | None = Field(default="high", description="Threat weight score for URL blocking.")    
    botnet_connection_detected: BotnetConnectionDetectedEnum | None = Field(default="critical", description="Threat weight score for detected botnet connections.")    
    malware: list[Malware] = Field(default=None, description="Anti-virus malware threat weight settings.")    
    ips: list[Ips] = Field(default=None, description="IPS threat weight settings.")    
    web: list[Web] = Field(default=None, description="Web filtering threat weight settings.")    
    geolocation: list[Geolocation] = Field(default=None, description="Geolocation-based threat weight settings.")    
    application: list[Application] = Field(default=None, description="Application-control threat weight settings.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ThreatWeightModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ThreatWeightModel",    "ThreatWeightLevel",    "ThreatWeightMalware",    "ThreatWeightIps",    "ThreatWeightWeb",    "ThreatWeightGeolocation",    "ThreatWeightApplication",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T04:31:43.018174Z
# ============================================================================


from typing import TypedDict, Literal, Any, Coroutine, Union, overload, Generator, final
from typing_extensions import NotRequired
from hfortix_fortios.models import FortiObject, FortiObjectList

# ============================================================================
# Nested TypedDicts for table field children (dict mode)
# These MUST be defined before the Payload class to use them as type hints
# ============================================================================

class GroupMemberItem(TypedDict, total=False):
    """Type hints for member table item fields (dict mode).
    
    Provides IDE autocomplete for nested table field items.
    Use this when building payloads for POST/PUT requests.
    
    **Available fields:**
        - name: str
    
    **Example:**
        entry: GroupMemberItem = {
            "status": "enable",  # <- autocomplete shows all fields and validates Literal values
        }
    """
    
    name: str  # Group member name. | MaxLen: 511


class GroupMatchItem(TypedDict, total=False):
    """Type hints for match table item fields (dict mode).
    
    Provides IDE autocomplete for nested table field items.
    Use this when building payloads for POST/PUT requests.
    
    **Available fields:**
        - id: int
        - server_name: str
        - group_name: str
    
    **Example:**
        entry: GroupMatchItem = {
            "status": "enable",  # <- autocomplete shows all fields and validates Literal values
        }
    """
    
    id: int  # ID. | Default: 0 | Min: 0 | Max: 4294967295
    server_name: str  # Name of remote auth server. | MaxLen: 35
    group_name: str  # Name of matching user or group on remote authentic | MaxLen: 511


class GroupGuestItem(TypedDict, total=False):
    """Type hints for guest table item fields (dict mode).
    
    Provides IDE autocomplete for nested table field items.
    Use this when building payloads for POST/PUT requests.
    
    **Available fields:**
        - id: int
        - user_id: str
        - name: str
        - password: str
        - mobile_phone: str
        - sponsor: str
        - company: str
        - email: str
        - expiration: str
        - comment: str
    
    **Example:**
        entry: GroupGuestItem = {
            "status": "enable",  # <- autocomplete shows all fields and validates Literal values
        }
    """
    
    id: int  # Guest ID. | Default: 0 | Min: 0 | Max: 4294967295
    user_id: str  # Guest ID. | MaxLen: 64
    name: str  # Guest name. | MaxLen: 64
    password: str  # Guest password. | MaxLen: 128
    mobile_phone: str  # Mobile phone. | MaxLen: 35
    sponsor: str  # Set the action for the sponsor guest user field. | MaxLen: 35
    company: str  # Set the action for the company guest user field. | MaxLen: 35
    email: str  # Email. | MaxLen: 64
    expiration: str  # Expire time.
    comment: str  # Comment. | MaxLen: 255


# ============================================================================
# Payload TypedDict for IDE autocomplete (for POST/PUT - fields are optional)
# ============================================================================
# NOTE: We intentionally DON'T use NotRequired wrapper because:
# 1. total=False already makes all fields optional
# 2. NotRequired[Literal[...]] prevents Pylance from validating Literal values in dict literals
class GroupPayload(TypedDict, total=False):
    """
    Type hints for user/group payload fields.
    
    Configure user groups.
    
    **Related Resources:**

    Dependencies (resources this endpoint references):
        - :class:`~.system.sms-server.SmsServerEndpoint` (via: sms-custom-server)

    **Usage:**
        payload: GroupPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    name: str  # Group name. | MaxLen: 35
    id: int  # Group ID. | Default: 0 | Min: 0 | Max: 4294967295
    group_type: Literal["firewall", "fsso-service", "rsso", "guest"]  # Set the group to be for firewall authentication, F | Default: firewall
    authtimeout: int  # Authentication timeout in minutes for this user gr | Default: 0 | Min: 0 | Max: 43200
    auth_concurrent_override: Literal["enable", "disable"]  # Enable/disable overriding the global number of con | Default: disable
    auth_concurrent_value: int  # Maximum number of concurrent authenticated connect | Default: 0 | Min: 0 | Max: 100
    http_digest_realm: str  # Realm attribute for MD5-digest authentication. | MaxLen: 35
    sso_attribute_value: str  # RADIUS attribute value. | MaxLen: 511
    member: list[GroupMemberItem]  # Names of users, peers, LDAP severs, RADIUS servers
    match: list[GroupMatchItem]  # Group matches.
    user_id: Literal["email", "auto-generate", "specify"]  # Guest user ID type. | Default: email
    password: Literal["auto-generate", "specify", "disable"]  # Guest user password type. | Default: auto-generate
    user_name: Literal["disable", "enable"]  # Enable/disable the guest user name entry. | Default: disable
    sponsor: Literal["optional", "mandatory", "disabled"]  # Set the action for the sponsor guest user field. | Default: optional
    company: Literal["optional", "mandatory", "disabled"]  # Set the action for the company guest user field. | Default: optional
    email: Literal["disable", "enable"]  # Enable/disable the guest user email address field. | Default: enable
    mobile_phone: Literal["disable", "enable"]  # Enable/disable the guest user mobile phone number | Default: disable
    sms_server: Literal["fortiguard", "custom"]  # Send SMS through FortiGuard or other external serv | Default: fortiguard
    sms_custom_server: str  # SMS server. | MaxLen: 35
    expire_type: Literal["immediately", "first-successful-login"]  # Determine when the expiration countdown begins. | Default: immediately
    expire: int  # Time in seconds before guest user accounts expire | Default: 14400 | Min: 1 | Max: 31536000
    max_accounts: int  # Maximum number of guest accounts that can be creat | Default: 0 | Min: 0 | Max: 500
    multiple_guest_add: Literal["disable", "enable"]  # Enable/disable addition of multiple guests. | Default: disable
    guest: list[GroupGuestItem]  # Guest User.

# ============================================================================
# Nested classes for table field children (object mode - for API responses)
# ============================================================================

@final
class GroupMemberObject:
    """Typed object for member table items.
    
    Provides IDE autocomplete for nested table field attributes.
    At runtime, this is a FortiObject instance.
    """
    
    # Group member name. | MaxLen: 511
    name: str
    
    # Common API response fields
    status: str
    http_status: int | None
    http_status_code: int | None
    http_method: str | None
    http_response_time: float | None
    vdom: str | None
    
    # Methods from FortiObject
    @property
    def dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        ...
    @property
    def json(self) -> str:
        """Get pretty-printed JSON string."""
        ...
    @property
    def raw(self) -> dict[str, Any]:
        """Get raw API response data."""
        ...
    def get_full(self, name: str) -> Any: ...
    def to_dict(self) -> FortiObject: ...
    def keys(self) -> Any: ...
    def values(self) -> Generator[Any, None, None]: ...
    def items(self) -> Generator[tuple[str, Any], None, None]: ...
    def get(self, key: str, default: Any = None) -> Any: ...


@final
class GroupMatchObject:
    """Typed object for match table items.
    
    Provides IDE autocomplete for nested table field attributes.
    At runtime, this is a FortiObject instance.
    """
    
    # ID. | Default: 0 | Min: 0 | Max: 4294967295
    id: int
    # Name of remote auth server. | MaxLen: 35
    server_name: str
    # Name of matching user or group on remote authentication serv | MaxLen: 511
    group_name: str
    
    # Common API response fields
    status: str
    http_status: int | None
    http_status_code: int | None
    http_method: str | None
    http_response_time: float | None
    vdom: str | None
    
    # Methods from FortiObject
    @property
    def dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        ...
    @property
    def json(self) -> str:
        """Get pretty-printed JSON string."""
        ...
    @property
    def raw(self) -> dict[str, Any]:
        """Get raw API response data."""
        ...
    def get_full(self, name: str) -> Any: ...
    def to_dict(self) -> FortiObject: ...
    def keys(self) -> Any: ...
    def values(self) -> Generator[Any, None, None]: ...
    def items(self) -> Generator[tuple[str, Any], None, None]: ...
    def get(self, key: str, default: Any = None) -> Any: ...


@final
class GroupGuestObject:
    """Typed object for guest table items.
    
    Provides IDE autocomplete for nested table field attributes.
    At runtime, this is a FortiObject instance.
    """
    
    # Guest ID. | Default: 0 | Min: 0 | Max: 4294967295
    id: int
    # Guest ID. | MaxLen: 64
    user_id: str
    # Guest name. | MaxLen: 64
    name: str
    # Guest password. | MaxLen: 128
    password: str
    # Mobile phone. | MaxLen: 35
    mobile_phone: str
    # Set the action for the sponsor guest user field. | MaxLen: 35
    sponsor: str
    # Set the action for the company guest user field. | MaxLen: 35
    company: str
    # Email. | MaxLen: 64
    email: str
    # Expire time.
    expiration: str
    # Comment. | MaxLen: 255
    comment: str
    
    # Common API response fields
    status: str
    http_status: int | None
    http_status_code: int | None
    http_method: str | None
    http_response_time: float | None
    vdom: str | None
    
    # Methods from FortiObject
    @property
    def dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        ...
    @property
    def json(self) -> str:
        """Get pretty-printed JSON string."""
        ...
    @property
    def raw(self) -> dict[str, Any]:
        """Get raw API response data."""
        ...
    def get_full(self, name: str) -> Any: ...
    def to_dict(self) -> FortiObject: ...
    def keys(self) -> Any: ...
    def values(self) -> Generator[Any, None, None]: ...
    def items(self) -> Generator[tuple[str, Any], None, None]: ...
    def get(self, key: str, default: Any = None) -> Any: ...




# Response TypedDict for GET returns (all fields present in API response)
class GroupResponse(TypedDict):
    """
    Type hints for user/group API response fields.
    
    All fields are present in the response from the FortiGate API.
    """
    name: str  # Group name. | MaxLen: 35
    id: int  # Group ID. | Default: 0 | Min: 0 | Max: 4294967295
    group_type: Literal["firewall", "fsso-service", "rsso", "guest"]  # Set the group to be for firewall authentication, F | Default: firewall
    authtimeout: int  # Authentication timeout in minutes for this user gr | Default: 0 | Min: 0 | Max: 43200
    auth_concurrent_override: Literal["enable", "disable"]  # Enable/disable overriding the global number of con | Default: disable
    auth_concurrent_value: int  # Maximum number of concurrent authenticated connect | Default: 0 | Min: 0 | Max: 100
    http_digest_realm: str  # Realm attribute for MD5-digest authentication. | MaxLen: 35
    sso_attribute_value: str  # RADIUS attribute value. | MaxLen: 511
    member: list[GroupMemberItem]  # Names of users, peers, LDAP severs, RADIUS servers
    match: list[GroupMatchItem]  # Group matches.
    user_id: Literal["email", "auto-generate", "specify"]  # Guest user ID type. | Default: email
    password: Literal["auto-generate", "specify", "disable"]  # Guest user password type. | Default: auto-generate
    user_name: Literal["disable", "enable"]  # Enable/disable the guest user name entry. | Default: disable
    sponsor: Literal["optional", "mandatory", "disabled"]  # Set the action for the sponsor guest user field. | Default: optional
    company: Literal["optional", "mandatory", "disabled"]  # Set the action for the company guest user field. | Default: optional
    email: Literal["disable", "enable"]  # Enable/disable the guest user email address field. | Default: enable
    mobile_phone: Literal["disable", "enable"]  # Enable/disable the guest user mobile phone number | Default: disable
    sms_server: Literal["fortiguard", "custom"]  # Send SMS through FortiGuard or other external serv | Default: fortiguard
    sms_custom_server: str  # SMS server. | MaxLen: 35
    expire_type: Literal["immediately", "first-successful-login"]  # Determine when the expiration countdown begins. | Default: immediately
    expire: int  # Time in seconds before guest user accounts expire | Default: 14400 | Min: 1 | Max: 31536000
    max_accounts: int  # Maximum number of guest accounts that can be creat | Default: 0 | Min: 0 | Max: 500
    multiple_guest_add: Literal["disable", "enable"]  # Enable/disable addition of multiple guests. | Default: disable
    guest: list[GroupGuestItem]  # Guest User.


@final
class GroupObject:
    """Typed FortiObject for user/group with IDE autocomplete support.
    
    This is a typed wrapper that provides IDE autocomplete for API response fields.
    At runtime, this is actually a FortiObject instance.
    """
    
    # Group name. | MaxLen: 35
    name: str
    # Group ID. | Default: 0 | Min: 0 | Max: 4294967295
    id: int
    # Set the group to be for firewall authentication, FSSO, RSSO, | Default: firewall
    group_type: Literal["firewall", "fsso-service", "rsso", "guest"]
    # Authentication timeout in minutes for this user group. 0 to | Default: 0 | Min: 0 | Max: 43200
    authtimeout: int
    # Enable/disable overriding the global number of concurrent au | Default: disable
    auth_concurrent_override: Literal["enable", "disable"]
    # Maximum number of concurrent authenticated connections per u | Default: 0 | Min: 0 | Max: 100
    auth_concurrent_value: int
    # Realm attribute for MD5-digest authentication. | MaxLen: 35
    http_digest_realm: str
    # RADIUS attribute value. | MaxLen: 511
    sso_attribute_value: str
    # Names of users, peers, LDAP severs, RADIUS servers or extern
    member: list[GroupMemberObject]
    # Group matches.
    match: list[GroupMatchObject]
    # Guest user ID type. | Default: email
    user_id: Literal["email", "auto-generate", "specify"]
    # Guest user password type. | Default: auto-generate
    password: Literal["auto-generate", "specify", "disable"]
    # Enable/disable the guest user name entry. | Default: disable
    user_name: Literal["disable", "enable"]
    # Set the action for the sponsor guest user field. | Default: optional
    sponsor: Literal["optional", "mandatory", "disabled"]
    # Set the action for the company guest user field. | Default: optional
    company: Literal["optional", "mandatory", "disabled"]
    # Enable/disable the guest user email address field. | Default: enable
    email: Literal["disable", "enable"]
    # Enable/disable the guest user mobile phone number field. | Default: disable
    mobile_phone: Literal["disable", "enable"]
    # Send SMS through FortiGuard or other external server. | Default: fortiguard
    sms_server: Literal["fortiguard", "custom"]
    # SMS server. | MaxLen: 35
    sms_custom_server: str
    # Determine when the expiration countdown begins. | Default: immediately
    expire_type: Literal["immediately", "first-successful-login"]
    # Time in seconds before guest user accounts expire | Default: 14400 | Min: 1 | Max: 31536000
    expire: int
    # Maximum number of guest accounts that can be created for thi | Default: 0 | Min: 0 | Max: 500
    max_accounts: int
    # Enable/disable addition of multiple guests. | Default: disable
    multiple_guest_add: Literal["disable", "enable"]
    # Guest User.
    guest: list[GroupGuestObject]
    
    # Common API response fields
    status: str
    http_status: int | None
    http_status_code: int | None
    http_method: str | None
    http_response_time: float | None
    vdom: str | None
    
    # Methods from FortiObject
    @property
    def dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        ...
    @property
    def json(self) -> str:
        """Get pretty-printed JSON string."""
        ...
    @property
    def raw(self) -> dict[str, Any]:
        """Get raw API response data."""
        ...
    def get_full(self, name: str) -> Any: ...
    def to_dict(self) -> GroupPayload: ...
    def keys(self) -> Any: ...
    def values(self) -> Generator[Any, None, None]: ...
    def items(self) -> Generator[tuple[str, Any], None, None]: ...
    def get(self, key: str, default: Any = None) -> Any: ...


class Group:
    """
    Configure user groups.
    
    Path: user/group
    Category: cmdb
    Primary Key: name
    """
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client.
        
        Args:
            client: HTTP client instance for API communication
        """
        ...
    
    # ================================================================
    # GET OVERLOADS - Always returns FortiObject
    # Pylance matches overloads top-to-bottom, so these must come first!
    # ================================================================
    
    # With mkey as positional arg -> returns FortiObject
    @overload
    def get(
        self,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject: ...
    
    # With mkey as keyword arg -> returns FortiObject
    @overload
    def get(
        self,
        *,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject: ...
    
    # Without mkey -> returns list of FortiObjects
    @overload
    def get(
        self,
        name: None = None,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObjectList[GroupObject]: ...
    
    # ================================================================
    # (removed - all GET now returns FortiObject)
    # ================================================================
    
    # With mkey as positional arg -> returns single object
    @overload
    def get(
        self,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject: ...
    
    # With mkey as keyword arg -> returns single object
    @overload
    def get(
        self,
        *,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject: ...
    
    # With no mkey -> returns list of objects
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObjectList[GroupObject]: ...
    
    # Dict mode with mkey provided as positional arg (single dict)
    @overload
    def get(
        self,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject: ...
    
    # Dict mode with mkey provided as keyword arg (single dict)
    @overload
    def get(
        self,
        *,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject: ...
    
    # Dict mode - list of dicts (no mkey/name provided) - keyword-only signature
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObjectList[GroupObject]: ...
    
    # Fallback overload for all other cases
    @overload
    def get(
        self,
        name: str | None = ...,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> Union[dict[str, Any], list[dict[str, Any]], FortiObject, list[FortiObject]]: ...
    
    def get(
        self,
        name: str | None = ...,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject | list[GroupObject] | dict[str, Any] | list[dict[str, Any]]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...
    
    # POST overloads
    @overload
    def post(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject: ...
    
    @overload
    def post(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # Default overload
    @overload
    def post(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    def post(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # PUT overloads
    @overload
    def put(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject: ...
    
    @overload
    def put(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # Default overload
    @overload
    def put(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    def put(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # DELETE overloads
    @overload
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> GroupObject: ...
    
    @overload
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # Default overload
    @overload
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


# ================================================================


__all__ = [
    "Group",
    "GroupPayload",
    "GroupResponse",
    "GroupObject",
]


from typing import TypedDict, Literal, Any, Coroutine, Union, overload, Generator, final
from typing_extensions import NotRequired
from hfortix_fortios.models import FortiObject, FortiObjectList

# ============================================================================
# Nested TypedDicts for table field children (dict mode)
# These MUST be defined before the Payload class to use them as type hints
# ============================================================================

class SettingAuthportsItem(TypedDict, total=False):
    """Type hints for auth-ports table item fields (dict mode).
    
    Provides IDE autocomplete for nested table field items.
    Use this when building payloads for POST/PUT requests.
    
    **Available fields:**
        - id: int
        - type: "http" | "https" | "ftp" | "telnet"
        - port: int
    
    **Example:**
        entry: SettingAuthportsItem = {
            "status": "enable",  # <- autocomplete shows all fields and validates Literal values
        }
    """
    
    id: int  # ID. | Default: 0 | Min: 0 | Max: 4294967295
    type: Literal["http", "https", "ftp", "telnet"]  # Service type. | Default: http
    port: int  # Non-standard port for firewall user authentication | Default: 1024 | Min: 1 | Max: 65535


class SettingCorsallowedoriginsItem(TypedDict, total=False):
    """Type hints for cors-allowed-origins table item fields (dict mode).
    
    Provides IDE autocomplete for nested table field items.
    Use this when building payloads for POST/PUT requests.
    
    **Available fields:**
        - name: str
    
    **Example:**
        entry: SettingCorsallowedoriginsItem = {
            "status": "enable",  # <- autocomplete shows all fields and validates Literal values
        }
    """
    
    name: str  # Allowed origin for CORS. | MaxLen: 79


# ============================================================================
# Payload TypedDict for IDE autocomplete (for POST/PUT - fields are optional)
# ============================================================================
# NOTE: We intentionally DON'T use NotRequired wrapper because:
# 1. total=False already makes all fields optional
# 2. NotRequired[Literal[...]] prevents Pylance from validating Literal values in dict literals
class SettingPayload(TypedDict, total=False):
    """
    Type hints for user/setting payload fields.
    
    Configure user authentication setting.
    
    **Related Resources:**

    Dependencies (resources this endpoint references):
        - :class:`~.user.password-policy.PasswordPolicyEndpoint` (via: default-user-password-policy)
        - :class:`~.vpn.certificate.local.LocalEndpoint` (via: auth-ca-cert, auth-cert)

    **Usage:**
        payload: SettingPayload = {
            "field": "value",  # <- autocomplete shows all fields
        }
    """
    auth_type: Literal["http", "https", "ftp", "telnet"]  # Supported firewall policy authentication protocols | Default: http https ftp telnet
    auth_cert: str  # HTTPS server certificate for policy authentication | MaxLen: 35
    auth_ca_cert: str  # HTTPS CA certificate for policy authentication. | MaxLen: 35
    auth_secure_http: Literal["enable", "disable"]  # Enable/disable redirecting HTTP user authenticatio | Default: disable
    auth_http_basic: Literal["enable", "disable"]  # Enable/disable use of HTTP basic authentication fo | Default: disable
    auth_ssl_allow_renegotiation: Literal["enable", "disable"]  # Allow/forbid SSL re-negotiation for HTTPS authenti | Default: disable
    auth_src_mac: Literal["enable", "disable"]  # Enable/disable source MAC for user identity. | Default: enable
    auth_on_demand: Literal["always", "implicitly"]  # Always/implicitly trigger firewall authentication | Default: implicitly
    auth_timeout: int  # Time in minutes before the firewall user authentic | Default: 5 | Min: 1 | Max: 1440
    auth_timeout_type: Literal["idle-timeout", "hard-timeout", "new-session"]  # Control if authenticated users have to login again | Default: idle-timeout
    auth_portal_timeout: int  # Time in minutes before captive portal user have to | Default: 3 | Min: 1 | Max: 30
    radius_ses_timeout_act: Literal["hard-timeout", "ignore-timeout"]  # Set the RADIUS session timeout to a hard timeout o | Default: hard-timeout
    auth_blackout_time: int  # Time in seconds an IP address is denied access aft | Default: 0 | Min: 0 | Max: 3600
    auth_invalid_max: int  # Maximum number of failed authentication attempts b | Default: 5 | Min: 1 | Max: 100
    auth_lockout_threshold: int  # Maximum number of failed login attempts before log | Default: 3 | Min: 1 | Max: 10
    auth_lockout_duration: int  # Lockout period in seconds after too many login fai | Default: 0 | Min: 0 | Max: 4294967295
    per_policy_disclaimer: Literal["enable", "disable"]  # Enable/disable per policy disclaimer. | Default: disable
    auth_ports: list[SettingAuthportsItem]  # Set up non-standard ports for authentication with
    auth_ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]  # Minimum supported protocol version for SSL/TLS con | Default: default
    auth_ssl_max_proto_version: Literal["sslv3", "tlsv1", "tlsv1-1", "tlsv1-2", "tlsv1-3"]  # Maximum supported protocol version for SSL/TLS con
    auth_ssl_sigalgs: Literal["no-rsa-pss", "all"]  # Set signature algorithms related to HTTPS authenti | Default: all
    default_user_password_policy: str  # Default password policy to apply to all local user | MaxLen: 35
    cors: Literal["disable", "enable"]  # Enable/disable allowed origins white list for CORS | Default: disable
    cors_allowed_origins: list[SettingCorsallowedoriginsItem]  # Allowed origins white list for CORS.

# ============================================================================
# Nested classes for table field children (object mode - for API responses)
# ============================================================================

@final
class SettingAuthportsObject:
    """Typed object for auth-ports table items.
    
    Provides IDE autocomplete for nested table field attributes.
    At runtime, this is a FortiObject instance.
    """
    
    # ID. | Default: 0 | Min: 0 | Max: 4294967295
    id: int
    # Service type. | Default: http
    type: Literal["http", "https", "ftp", "telnet"]
    # Non-standard port for firewall user authentication. | Default: 1024 | Min: 1 | Max: 65535
    port: int
    
    # Common API response fields
    status: str
    http_status: int | None
    http_status_code: int | None
    http_method: str | None
    http_response_time: float | None
    vdom: str | None
    
    # Methods from FortiObject
    @property
    def dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        ...
    @property
    def json(self) -> str:
        """Get pretty-printed JSON string."""
        ...
    @property
    def raw(self) -> dict[str, Any]:
        """Get raw API response data."""
        ...
    def get_full(self, name: str) -> Any: ...
    def to_dict(self) -> FortiObject: ...
    def keys(self) -> Any: ...
    def values(self) -> Generator[Any, None, None]: ...
    def items(self) -> Generator[tuple[str, Any], None, None]: ...
    def get(self, key: str, default: Any = None) -> Any: ...


@final
class SettingCorsallowedoriginsObject:
    """Typed object for cors-allowed-origins table items.
    
    Provides IDE autocomplete for nested table field attributes.
    At runtime, this is a FortiObject instance.
    """
    
    # Allowed origin for CORS. | MaxLen: 79
    name: str
    
    # Common API response fields
    status: str
    http_status: int | None
    http_status_code: int | None
    http_method: str | None
    http_response_time: float | None
    vdom: str | None
    
    # Methods from FortiObject
    @property
    def dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        ...
    @property
    def json(self) -> str:
        """Get pretty-printed JSON string."""
        ...
    @property
    def raw(self) -> dict[str, Any]:
        """Get raw API response data."""
        ...
    def get_full(self, name: str) -> Any: ...
    def to_dict(self) -> FortiObject: ...
    def keys(self) -> Any: ...
    def values(self) -> Generator[Any, None, None]: ...
    def items(self) -> Generator[tuple[str, Any], None, None]: ...
    def get(self, key: str, default: Any = None) -> Any: ...




# Response TypedDict for GET returns (all fields present in API response)
class SettingResponse(TypedDict):
    """
    Type hints for user/setting API response fields.
    
    All fields are present in the response from the FortiGate API.
    """
    auth_type: Literal["http", "https", "ftp", "telnet"]  # Supported firewall policy authentication protocols | Default: http https ftp telnet
    auth_cert: str  # HTTPS server certificate for policy authentication | MaxLen: 35
    auth_ca_cert: str  # HTTPS CA certificate for policy authentication. | MaxLen: 35
    auth_secure_http: Literal["enable", "disable"]  # Enable/disable redirecting HTTP user authenticatio | Default: disable
    auth_http_basic: Literal["enable", "disable"]  # Enable/disable use of HTTP basic authentication fo | Default: disable
    auth_ssl_allow_renegotiation: Literal["enable", "disable"]  # Allow/forbid SSL re-negotiation for HTTPS authenti | Default: disable
    auth_src_mac: Literal["enable", "disable"]  # Enable/disable source MAC for user identity. | Default: enable
    auth_on_demand: Literal["always", "implicitly"]  # Always/implicitly trigger firewall authentication | Default: implicitly
    auth_timeout: int  # Time in minutes before the firewall user authentic | Default: 5 | Min: 1 | Max: 1440
    auth_timeout_type: Literal["idle-timeout", "hard-timeout", "new-session"]  # Control if authenticated users have to login again | Default: idle-timeout
    auth_portal_timeout: int  # Time in minutes before captive portal user have to | Default: 3 | Min: 1 | Max: 30
    radius_ses_timeout_act: Literal["hard-timeout", "ignore-timeout"]  # Set the RADIUS session timeout to a hard timeout o | Default: hard-timeout
    auth_blackout_time: int  # Time in seconds an IP address is denied access aft | Default: 0 | Min: 0 | Max: 3600
    auth_invalid_max: int  # Maximum number of failed authentication attempts b | Default: 5 | Min: 1 | Max: 100
    auth_lockout_threshold: int  # Maximum number of failed login attempts before log | Default: 3 | Min: 1 | Max: 10
    auth_lockout_duration: int  # Lockout period in seconds after too many login fai | Default: 0 | Min: 0 | Max: 4294967295
    per_policy_disclaimer: Literal["enable", "disable"]  # Enable/disable per policy disclaimer. | Default: disable
    auth_ports: list[SettingAuthportsItem]  # Set up non-standard ports for authentication with
    auth_ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]  # Minimum supported protocol version for SSL/TLS con | Default: default
    auth_ssl_max_proto_version: Literal["sslv3", "tlsv1", "tlsv1-1", "tlsv1-2", "tlsv1-3"]  # Maximum supported protocol version for SSL/TLS con
    auth_ssl_sigalgs: Literal["no-rsa-pss", "all"]  # Set signature algorithms related to HTTPS authenti | Default: all
    default_user_password_policy: str  # Default password policy to apply to all local user | MaxLen: 35
    cors: Literal["disable", "enable"]  # Enable/disable allowed origins white list for CORS | Default: disable
    cors_allowed_origins: list[SettingCorsallowedoriginsItem]  # Allowed origins white list for CORS.


@final
class SettingObject:
    """Typed FortiObject for user/setting with IDE autocomplete support.
    
    This is a typed wrapper that provides IDE autocomplete for API response fields.
    At runtime, this is actually a FortiObject instance.
    """
    
    # Supported firewall policy authentication protocols/methods. | Default: http https ftp telnet
    auth_type: Literal["http", "https", "ftp", "telnet"]
    # HTTPS server certificate for policy authentication. | MaxLen: 35
    auth_cert: str
    # HTTPS CA certificate for policy authentication. | MaxLen: 35
    auth_ca_cert: str
    # Enable/disable redirecting HTTP user authentication to more | Default: disable
    auth_secure_http: Literal["enable", "disable"]
    # Enable/disable use of HTTP basic authentication for identity | Default: disable
    auth_http_basic: Literal["enable", "disable"]
    # Allow/forbid SSL re-negotiation for HTTPS authentication. | Default: disable
    auth_ssl_allow_renegotiation: Literal["enable", "disable"]
    # Enable/disable source MAC for user identity. | Default: enable
    auth_src_mac: Literal["enable", "disable"]
    # Always/implicitly trigger firewall authentication on demand. | Default: implicitly
    auth_on_demand: Literal["always", "implicitly"]
    # Time in minutes before the firewall user authentication time | Default: 5 | Min: 1 | Max: 1440
    auth_timeout: int
    # Control if authenticated users have to login again after a h | Default: idle-timeout
    auth_timeout_type: Literal["idle-timeout", "hard-timeout", "new-session"]
    # Time in minutes before captive portal user have to re-authen | Default: 3 | Min: 1 | Max: 30
    auth_portal_timeout: int
    # Set the RADIUS session timeout to a hard timeout or to ignor | Default: hard-timeout
    radius_ses_timeout_act: Literal["hard-timeout", "ignore-timeout"]
    # Time in seconds an IP address is denied access after failing | Default: 0 | Min: 0 | Max: 3600
    auth_blackout_time: int
    # Maximum number of failed authentication attempts before the | Default: 5 | Min: 1 | Max: 100
    auth_invalid_max: int
    # Maximum number of failed login attempts before login lockout | Default: 3 | Min: 1 | Max: 10
    auth_lockout_threshold: int
    # Lockout period in seconds after too many login failures. | Default: 0 | Min: 0 | Max: 4294967295
    auth_lockout_duration: int
    # Enable/disable per policy disclaimer. | Default: disable
    per_policy_disclaimer: Literal["enable", "disable"]
    # Set up non-standard ports for authentication with HTTP, HTTP
    auth_ports: list[SettingAuthportsObject]
    # Minimum supported protocol version for SSL/TLS connections | Default: default
    auth_ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]
    # Maximum supported protocol version for SSL/TLS connections
    auth_ssl_max_proto_version: Literal["sslv3", "tlsv1", "tlsv1-1", "tlsv1-2", "tlsv1-3"]
    # Set signature algorithms related to HTTPS authentication | Default: all
    auth_ssl_sigalgs: Literal["no-rsa-pss", "all"]
    # Default password policy to apply to all local users unless o | MaxLen: 35
    default_user_password_policy: str
    # Enable/disable allowed origins white list for CORS. | Default: disable
    cors: Literal["disable", "enable"]
    # Allowed origins white list for CORS.
    cors_allowed_origins: list[SettingCorsallowedoriginsObject]
    
    # Common API response fields
    status: str
    http_status: int | None
    http_status_code: int | None
    http_method: str | None
    http_response_time: float | None
    vdom: str | None
    
    # Methods from FortiObject
    @property
    def dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        ...
    @property
    def json(self) -> str:
        """Get pretty-printed JSON string."""
        ...
    @property
    def raw(self) -> dict[str, Any]:
        """Get raw API response data."""
        ...
    def get_full(self, name: str) -> Any: ...
    def to_dict(self) -> SettingPayload: ...
    def keys(self) -> Any: ...
    def values(self) -> Generator[Any, None, None]: ...
    def items(self) -> Generator[tuple[str, Any], None, None]: ...
    def get(self, key: str, default: Any = None) -> Any: ...


class Setting:
    """
    Configure user authentication setting.
    
    Path: user/setting
    Category: cmdb
    """
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client.
        
        Args:
            client: HTTP client instance for API communication
        """
        ...
    
    # ================================================================
    # GET OVERLOADS - Always returns FortiObject
    # Pylance matches overloads top-to-bottom, so these must come first!
    # ================================================================
    
    # With mkey as positional arg -> returns FortiObject
    @overload
    def get(
        self,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # With mkey as keyword arg -> returns FortiObject
    @overload
    def get(
        self,
        *,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Without mkey -> returns list of FortiObjects
    @overload
    def get(
        self,
        name: None = None,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # ================================================================
    # (removed - all GET now returns FortiObject)
    # ================================================================
    
    # With mkey as positional arg -> returns single object
    @overload
    def get(
        self,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # With mkey as keyword arg -> returns single object
    @overload
    def get(
        self,
        *,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # With no mkey -> returns list of objects
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Dict mode with mkey provided as positional arg (single dict)
    @overload
    def get(
        self,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Dict mode with mkey provided as keyword arg (single dict)
    @overload
    def get(
        self,
        *,
        name: str,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Dict mode - list of dicts (no mkey/name provided) - keyword-only signature
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    # Fallback overload for all other cases
    @overload
    def get(
        self,
        name: str | None = ...,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> dict[str, Any] | FortiObject: ...
    
    def get(
        self,
        name: str | None = ...,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject | dict[str, Any]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...
    
    # PUT overloads
    @overload
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        auth_type: Literal["http", "https", "ftp", "telnet"] | list[str] | None = ...,
        auth_cert: str | None = ...,
        auth_ca_cert: str | None = ...,
        auth_secure_http: Literal["enable", "disable"] | None = ...,
        auth_http_basic: Literal["enable", "disable"] | None = ...,
        auth_ssl_allow_renegotiation: Literal["enable", "disable"] | None = ...,
        auth_src_mac: Literal["enable", "disable"] | None = ...,
        auth_on_demand: Literal["always", "implicitly"] | None = ...,
        auth_timeout: int | None = ...,
        auth_timeout_type: Literal["idle-timeout", "hard-timeout", "new-session"] | None = ...,
        auth_portal_timeout: int | None = ...,
        radius_ses_timeout_act: Literal["hard-timeout", "ignore-timeout"] | None = ...,
        auth_blackout_time: int | None = ...,
        auth_invalid_max: int | None = ...,
        auth_lockout_threshold: int | None = ...,
        auth_lockout_duration: int | None = ...,
        per_policy_disclaimer: Literal["enable", "disable"] | None = ...,
        auth_ports: str | list[str] | list[SettingAuthportsItem] | None = ...,
        auth_ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        auth_ssl_max_proto_version: Literal["sslv3", "tlsv1", "tlsv1-1", "tlsv1-2", "tlsv1-3"] | None = ...,
        auth_ssl_sigalgs: Literal["no-rsa-pss", "all"] | None = ...,
        default_user_password_policy: str | None = ...,
        cors: Literal["disable", "enable"] | None = ...,
        cors_allowed_origins: str | list[str] | list[SettingCorsallowedoriginsItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> SettingObject: ...
    
    @overload
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        auth_type: Literal["http", "https", "ftp", "telnet"] | list[str] | None = ...,
        auth_cert: str | None = ...,
        auth_ca_cert: str | None = ...,
        auth_secure_http: Literal["enable", "disable"] | None = ...,
        auth_http_basic: Literal["enable", "disable"] | None = ...,
        auth_ssl_allow_renegotiation: Literal["enable", "disable"] | None = ...,
        auth_src_mac: Literal["enable", "disable"] | None = ...,
        auth_on_demand: Literal["always", "implicitly"] | None = ...,
        auth_timeout: int | None = ...,
        auth_timeout_type: Literal["idle-timeout", "hard-timeout", "new-session"] | None = ...,
        auth_portal_timeout: int | None = ...,
        radius_ses_timeout_act: Literal["hard-timeout", "ignore-timeout"] | None = ...,
        auth_blackout_time: int | None = ...,
        auth_invalid_max: int | None = ...,
        auth_lockout_threshold: int | None = ...,
        auth_lockout_duration: int | None = ...,
        per_policy_disclaimer: Literal["enable", "disable"] | None = ...,
        auth_ports: str | list[str] | list[SettingAuthportsItem] | None = ...,
        auth_ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        auth_ssl_max_proto_version: Literal["sslv3", "tlsv1", "tlsv1-1", "tlsv1-2", "tlsv1-3"] | None = ...,
        auth_ssl_sigalgs: Literal["no-rsa-pss", "all"] | None = ...,
        default_user_password_policy: str | None = ...,
        cors: Literal["disable", "enable"] | None = ...,
        cors_allowed_origins: str | list[str] | list[SettingCorsallowedoriginsItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # Default overload
    @overload
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        auth_type: Literal["http", "https", "ftp", "telnet"] | list[str] | None = ...,
        auth_cert: str | None = ...,
        auth_ca_cert: str | None = ...,
        auth_secure_http: Literal["enable", "disable"] | None = ...,
        auth_http_basic: Literal["enable", "disable"] | None = ...,
        auth_ssl_allow_renegotiation: Literal["enable", "disable"] | None = ...,
        auth_src_mac: Literal["enable", "disable"] | None = ...,
        auth_on_demand: Literal["always", "implicitly"] | None = ...,
        auth_timeout: int | None = ...,
        auth_timeout_type: Literal["idle-timeout", "hard-timeout", "new-session"] | None = ...,
        auth_portal_timeout: int | None = ...,
        radius_ses_timeout_act: Literal["hard-timeout", "ignore-timeout"] | None = ...,
        auth_blackout_time: int | None = ...,
        auth_invalid_max: int | None = ...,
        auth_lockout_threshold: int | None = ...,
        auth_lockout_duration: int | None = ...,
        per_policy_disclaimer: Literal["enable", "disable"] | None = ...,
        auth_ports: str | list[str] | list[SettingAuthportsItem] | None = ...,
        auth_ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        auth_ssl_max_proto_version: Literal["sslv3", "tlsv1", "tlsv1-1", "tlsv1-2", "tlsv1-3"] | None = ...,
        auth_ssl_sigalgs: Literal["no-rsa-pss", "all"] | None = ...,
        default_user_password_policy: str | None = ...,
        cors: Literal["disable", "enable"] | None = ...,
        cors_allowed_origins: str | list[str] | list[SettingCorsallowedoriginsItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        auth_type: Literal["http", "https", "ftp", "telnet"] | list[str] | None = ...,
        auth_cert: str | None = ...,
        auth_ca_cert: str | None = ...,
        auth_secure_http: Literal["enable", "disable"] | None = ...,
        auth_http_basic: Literal["enable", "disable"] | None = ...,
        auth_ssl_allow_renegotiation: Literal["enable", "disable"] | None = ...,
        auth_src_mac: Literal["enable", "disable"] | None = ...,
        auth_on_demand: Literal["always", "implicitly"] | None = ...,
        auth_timeout: int | None = ...,
        auth_timeout_type: Literal["idle-timeout", "hard-timeout", "new-session"] | None = ...,
        auth_portal_timeout: int | None = ...,
        radius_ses_timeout_act: Literal["hard-timeout", "ignore-timeout"] | None = ...,
        auth_blackout_time: int | None = ...,
        auth_invalid_max: int | None = ...,
        auth_lockout_threshold: int | None = ...,
        auth_lockout_duration: int | None = ...,
        per_policy_disclaimer: Literal["enable", "disable"] | None = ...,
        auth_ports: str | list[str] | list[SettingAuthportsItem] | None = ...,
        auth_ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        auth_ssl_max_proto_version: Literal["sslv3", "tlsv1", "tlsv1-1", "tlsv1-2", "tlsv1-3"] | None = ...,
        auth_ssl_sigalgs: Literal["no-rsa-pss", "all"] | None = ...,
        default_user_password_policy: str | None = ...,
        cors: Literal["disable", "enable"] | None = ...,
        cors_allowed_origins: str | list[str] | list[SettingCorsallowedoriginsItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: SettingPayload | None = ...,
        auth_type: Literal["http", "https", "ftp", "telnet"] | list[str] | None = ...,
        auth_cert: str | None = ...,
        auth_ca_cert: str | None = ...,
        auth_secure_http: Literal["enable", "disable"] | None = ...,
        auth_http_basic: Literal["enable", "disable"] | None = ...,
        auth_ssl_allow_renegotiation: Literal["enable", "disable"] | None = ...,
        auth_src_mac: Literal["enable", "disable"] | None = ...,
        auth_on_demand: Literal["always", "implicitly"] | None = ...,
        auth_timeout: int | None = ...,
        auth_timeout_type: Literal["idle-timeout", "hard-timeout", "new-session"] | None = ...,
        auth_portal_timeout: int | None = ...,
        radius_ses_timeout_act: Literal["hard-timeout", "ignore-timeout"] | None = ...,
        auth_blackout_time: int | None = ...,
        auth_invalid_max: int | None = ...,
        auth_lockout_threshold: int | None = ...,
        auth_lockout_duration: int | None = ...,
        per_policy_disclaimer: Literal["enable", "disable"] | None = ...,
        auth_ports: str | list[str] | list[SettingAuthportsItem] | None = ...,
        auth_ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        auth_ssl_max_proto_version: Literal["sslv3", "tlsv1", "tlsv1-1", "tlsv1-2", "tlsv1-3"] | None = ...,
        auth_ssl_sigalgs: Literal["no-rsa-pss", "all"] | None = ...,
        default_user_password_policy: str | None = ...,
        cors: Literal["disable", "enable"] | None = ...,
        cors_allowed_origins: str | list[str] | list[SettingCorsallowedoriginsItem] | None = ...,
        vdom: str | bool | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> Union[list[str], list[dict[str, Any]]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


# ================================================================


__all__ = [
    "Setting",
    "SettingPayload",
    "SettingResponse",
    "SettingObject",
]